@@ -15,6 +15,13 @@ logger = logging.getLogger(__name__)
 
 def read_and_parse_input_data(filename, normal_name='diploid', input_type='tsv', separator='X', 
                               allele_columns=['cn_a', 'cn_b'], maxcn=8, total_copy_numbers=False):
+
+    if len(allele_columns) == 1 and not total_copy_numbers:
+        logger.warn('You have provided only one allele column but the --total-copy-numbers flag was not set')
+    if total_copy_numbers and not len(allele_columns) == 1:
+        raise MEDICCIOError("You have set the --total-copy-numbers flag but provided more than one allele column. "
+                            "Set allele columns with the flag --input-allele-columns")
+
     ## Read in input data
     if input_type.lower() == "fasta" or input_type.lower() == 'f':
         logger.info("Reading FASTA input.")
@@ -25,12 +32,20 @@ def read_and_parse_input_data(filename, normal_name='diploid', input_type='tsv',
     else:
         raise MEDICCIOError("Unknown input type, possible options are 'fasta' or 'tsv'.")
 
-    if len(allele_columns) == 1 and not total_copy_numbers:
-        logger.warn('You have provided only one allele column but the --total-copy-numbers flag was not set')
-    if total_copy_numbers and not len(allele_columns) == 1:
-        raise MEDICCIOError("You have set the --total-copy-numbers flag but provided more than one allele column. "
-                            "Set allele columns with the flag --input-allele-columns")
+    input_df.columns.name = 'allele'
+    input_df_stacked = input_df.stack('allele').unstack('sample_id').T
 
+    duplicated_entries = input_df_stacked.duplicated(keep=False)
+    if duplicated_entries.any():
+        logger.warn("Duplicated entries found in input data: "
+                    f"{input_df_stacked.index[duplicated_entries]}")
+
+    normal_value = '2' if total_copy_numbers else '1'
+    normal_samples = np.setdiff1d(input_df_stacked.index[
+        (input_df_stacked == normal_value).all(axis=1)], normal_name)
+    if len(normal_samples) > 0:
+        logger.warn(f"Normal samples found in input data: {normal_samples}")
+    
     ## Add normal sample if needed
     input_df = add_normal_sample(input_df, normal_name, allele_columns=allele_columns, 
                                     total_copy_numbers=total_copy_numbers)
@@ -38,23 +53,38 @@ def read_and_parse_input_data(filename, normal_name='diploid', input_type='tsv',
     nchr = input_df.index.get_level_values('chrom').unique().shape[0]
     nsegs = input_df.loc[normal_name,:].shape[0]
     logger.info("Read %d samples, %d chromosomes, %d segments per sample", nsamples, nchr, nsegs)
+
+    gaps = (input_df.loc[normal_name].eval('start') - 
+            np.roll(input_df.loc[normal_name].eval('end'), 1)).values
+    total_gaps = gaps[gaps>0].sum()
+    if total_gaps > 1e8:
+        logger.warn(f"Total of {total_gaps:.1e} bp gaps in the segmentation. Large gaps might "
+                    "affect the performance of MEDICC2.")
+
     return input_df
 
 
-def read_fst(user_fst=None, no_wgd=False, n_wgd=None):
-    """ Simple wrapper for loading the FST using the fstlib read function. """
+def read_fst(user_fst=None, no_wgd=False, n_wgd=None, total_copy_numbers=False, wgd_x2=False):
+    """Simple wrapper for loading the FST using the fstlib read function. """
 
     objects_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "objects")
 
     if user_fst is not None:
         fst_path = user_fst
+    elif no_wgd:
+        fst_path = os.path.join(objects_dir, 'no_wgd_asymm.fst')
+    elif wgd_x2:
+        fst_path = os.path.join(objects_dir, 'wgd_x2_asymm.fst')
+        if n_wgd == 1:
+            fst_path = os.path.join(objects_dir, 'wgd_x2_1_asymm.fst')
+    elif total_copy_numbers:
+        fst_path = os.path.join(objects_dir, 'wgd_total_cn_asymm.fst')
+        if n_wgd == 1:
+            fst_path = os.path.join(objects_dir, 'wgd_total_cn_1_asymm.fst')
+    elif n_wgd is not None and int(n_wgd) <= 3:
+        fst_path = os.path.join(objects_dir, 'wgd_{}_asymm.fst'.format(int(n_wgd)))
     else:
-        if no_wgd:
-            fst_path = os.path.join(objects_dir, 'no_wgd_asymm.fst')
-        elif n_wgd is not None and int(n_wgd) <= 3:
-            fst_path = os.path.join(objects_dir, 'wgd_{}_asymm.fst'.format(int(n_wgd)))
-        else:
-            fst_path = os.path.join(objects_dir, 'wgd_asymm.fst')
+        fst_path = os.path.join(objects_dir, 'wgd_asymm.fst')
 
     return fstlib.read(fst_path)
 
@@ -63,57 +93,76 @@ def load_main_fsts(return_symbol_table=False):
     asymm_fst = read_fst()
     asymm_fst_nowgd = read_fst(no_wgd=True)
     asymm_fst_1_wgd = read_fst(n_wgd=1)
+    asymm_fst_2_wgd = read_fst(n_wgd=2)
 
     if return_symbol_table:
         symbol_table = asymm_fst.input_symbols()
-        return asymm_fst, asymm_fst_nowgd, asymm_fst_1_wgd, symbol_table
+        return asymm_fst, asymm_fst_nowgd, asymm_fst_1_wgd, asymm_fst_2_wgd, symbol_table
     else:
-        return asymm_fst, asymm_fst_nowgd, asymm_fst_1_wgd
+        return asymm_fst, asymm_fst_nowgd, asymm_fst_1_wgd, asymm_fst_2_wgd
 
 
-def validate_input(input_df, symbol_table):
+def validate_input(input_df, symbol_table=None, normal_name='diploid'):
+    """Validate the input DataFrame."""
+
+    # Check that normal sample is present
+    if normal_name not in input_df.index.get_level_values('sample_id').unique():
+        raise MEDICCIOError(f"Normal sample '{normal_name}' not found in input data. Specify a "
+                            "different name using the --normal-name flag.")
+
+    # Check if the index names are correct
+    if input_df.index.names != ['sample_id', 'chrom', 'start', 'end']:
+        raise MEDICCIOError("DataFrame must be indexed by ['sample_id', 'chrom', 'start', 'end'].")
+
+    # Check if the chromosome is categorical
+    if not pd.api.types.is_categorical_dtype(input_df.index.dtypes['chrom']):
+        raise MEDICCIOError("""
+            Chromosome index 'chrom' must be of type pd.Categorical. 
+            You can use medicc.tools.format_chromosomes() or create it yourself.""")
+
+    # Check if the index is sorted
+    if not input_df.index.is_monotonic_increasing:
+        raise MEDICCIOError("DataFrame index must be sorted.")
+
     # Check the number of alleles
     if len(input_df.columns)>2:
         raise MEDICCIOError("More than 2 alleles are currently not supported.")
 
+    # Check if there are any alleles
     if len(input_df.columns)==0:
         raise MEDICCIOError("No alleles found.")
-
-    # Check if index of dataframe is sorted
-    if not input_df.index.is_monotonic_increasing:
-        raise MEDICCIOError("DataFrame index must be sorted.")
-
-    # Check if all samples have same number of segments
-    if input_df.unstack('sample_id').isna().sum().sum() != 0:
-        raise MEDICCIOError("The samples have different number of segments!\n"
-                            "Total number of unique segments: {}\n".format(len(input_df.unstack('sample_id'))))
-
-    # Check if symbols are in symbol table
-    alphabet = {x[1] for x in symbol_table}
-    data_chars = set(input_df.values.flatten())
-    if not data_chars.issubset(alphabet):
-        not_in_set = data_chars.difference(alphabet)
-        raise MEDICCIOError("Not all input symbols are contained in symbol table. Offending symbols: %s" % str(not_in_set))
+    
+    # Check data type allele columns - these should all be of type str (object)
+    if not np.all([pd.api.types.is_string_dtype(x) for x in input_df.dtypes]):
+        raise MEDICCIOError("Allele columns must be of type: string.")
 
     # Check data type start and end columns
-    if (input_df.index.get_level_values('start').dtype != np.int or 
-        input_df.index.get_level_values('end').dtype != np.int):
+    if (input_df.index.get_level_values('start').dtype != int or 
+        input_df.index.get_level_values('end').dtype != int):
         raise MEDICCIOError("Start and end columns must be of type: integer.")
 
-    # Check data type payload columns - these should all be of type str (object)
-    if not np.all([pd.api.types.is_string_dtype(x) for x in input_df.dtypes]):
-        raise MEDICCIOError("Payload columns must be of type: string.")
+    # Check if all samples have same segments
+    if input_df.unstack('sample_id').isna().sum().sum() != 0:
+        raise MEDICCIOError("The samples have different segments!\n"
+                            "Total number of unique segments: {}\n".format(len(input_df.unstack('sample_id'))))
+
+    if symbol_table is not None:
+        # Check if symbols are in symbol table
+        alphabet = {x[1] for x in symbol_table}
+        data_chars = set(input_df.values.flatten())
+        if not data_chars.issubset(alphabet):
+            not_in_set = data_chars.difference(alphabet)
+            raise MEDICCIOError("Not all input symbols are contained in symbol table. Offending symbols: %s" % str(not_in_set))
 
-    # Check if index of dataframe is sorted
-    if not input_df.index.is_monotonic_increasing:
-        raise MEDICCIOError("DataFrame index must be sorted.")
 
     logger.info('Input data is valid!')
 
+
 def filter_by_segment_length(input_df, filter_size):
-    segment_length = input_df.eval('end-start')
+    segment_length = input_df.eval('end+1-start')
     return input_df.loc[segment_length > float(filter_size)]
 
+
 def _read_tsv_as_dataframe(path, allele_columns=['cn_a','cn_b'], maxcn=8):
     logger.info("Reading TSV file %s", path)
     input_file = pd.read_csv(path, sep = "\t")
@@ -142,8 +191,9 @@ def _read_tsv_as_dataframe(path, allele_columns=['cn_a','cn_b'], maxcn=8):
 
     return input_file
 
+
 def _read_fasta_as_dataframe(infile: str, separator: str = 'X', allele_columns = ['cn_a','cn_b'], maxcn: int = 8):
-    """ Reads FASTA decriptor file (old MEDICC input format) and reads the corresponding FASTA files to generate
+    """Reads FASTA decriptor file (old MEDICC input format) and reads the corresponding FASTA files to generate
     a data frame with the same format as the refphase input (TSV) format. """
     logger.info("Reading FASTA dataset from description file %s.", infile)
     description_file = pd.read_csv(infile,
@@ -200,7 +250,7 @@ def _read_fasta_as_dataframe(infile: str, separator: str = 'X', allele_columns =
     return result
 
 def add_normal_sample(df, normal_name, allele_columns=['cn_a','cn_b'], total_copy_numbers=False):
-    """ Adds an artificial normal samples with the supplied name to the data frame.
+    """Adds an artificial normal samples with the supplied name to the data frame.
     The normal sample has CN=1 on all supplied alleles. """
     samples = df.index.get_level_values('sample_id').unique()
 
@@ -229,8 +279,8 @@ def add_normal_sample(df, normal_name, allele_columns=['cn_a','cn_b'], total_cop
     return tmp
 
 
-def write_tree_files(tree, out_name: str, plot_tree=True, draw_ascii=False):
-    """ Writes a Newick, PhyloXML, Ascii graphic and PNG grahic file of the tree. """
+def write_tree_files(tree, out_name: str, plot_tree=True, draw_ascii=False, normal_name='diploid'):
+    """Writes a Newick, PhyloXML, Ascii graphic and PNG grahic file of the tree. """
     Bio.Phylo.write(tree, out_name + ".new", "newick")
     Bio.Phylo.write(tree, out_name + ".xml", "phyloxml")
 
@@ -241,11 +291,21 @@ def write_tree_files(tree, out_name: str, plot_tree=True, draw_ascii=False):
     if plot_tree:
         plot.plot_tree(tree,
                        output_name=out_name,
+                       normal_name=normal_name,
                        label_func=lambda x: x if 'internal' not in x else '',
                        show_branch_lengths=True)
 
+
+def write_branch_lengths(tree, out_name: str):
+    """Writes a file with the branch lengths of the tree."""
+    with open(out_name, "w") as f:
+        for node in tree.find_clades():
+            if node.name is not None and node.name != 'diploid':
+                f.write("{}\t{}\n".format(node.name, node.branch_length))
+
+
 def write_pairwise_distances(sample_labels, pairwise_distances, filename_prefix):
-    """ Write the pairwise distance matrix as a tsv."""
+    """Write the pairwise distance matrix as a tsv."""
 
     if not isinstance(pairwise_distances, pd.DataFrame):
         pairwise_distances = pd.DataFrame(pairwise_distances, columns=sample_labels, index=sample_labels)
@@ -253,7 +313,7 @@ def write_pairwise_distances(sample_labels, pairwise_distances, filename_prefix)
 
 
 def import_tree(tree_file, normal_name='diploid', file_format='newick'):
-    """ Loads a phylogenetic tree in the given format and roots it at the normal sample. """
+    """Loads a phylogenetic tree in the given format and roots it at the normal sample. """
     tree = Bio.Phylo.read(tree_file, file_format)
     input_tree = Bio.Phylo.BaseTree.copy.deepcopy(tree)
     tmpsearch = [c for c in input_tree.find_clades(name = normal_name)]
@@ -266,6 +326,11 @@ def import_tree(tree_file, normal_name='diploid', file_format='newick'):
     else:
         pass
 
+    # check that internal node names are unique
+    node_names = [c.name for c in input_tree.find_clades() if c.name is not None]
+    if len(node_names) != len(np.unique(node_names)):
+        logger.warning("Internal node names are not unique. This will cause problems in MEDICC2.")
+
     return input_tree
 
 
