@@ -6,8 +6,6 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 
-from medicc import core
-
 logger = logging.getLogger(__name__)
 
 COL_ALLELE_A = mpl.colors.to_rgba('orange')
@@ -72,6 +70,7 @@ def plot_cn_profiles(
         label_func=None):
     
     df = input_df.copy()
+    df[allele_columns] = df[allele_columns].astype(int)
     if len(allele_columns) > 2:
         logger.warn("More than two allels were provided ({})\n"
                     "Copy number tracks can only be plotted for 1 or 2 alleles".format(allele_columns))
@@ -80,7 +79,34 @@ def plot_cn_profiles(
                     "These are: {}".format(np.setdiff1d(allele_columns, df.columns)))
 
     if np.setdiff1d(['is_clonal', 'is_normal', 'is_gain', 'is_loss', 'is_wgd'], df.columns).size > 0:
-        df, _ = core.calculate_all_cn_events(input_tree, df, allele_columns, normal_name)
+        if input_tree is None:
+            df[['is_normal', 'is_clonal', 'is_gain', 'is_loss', 'is_wgd']] = False
+        else:
+            df[['is_gain', 'is_loss', 'is_wgd']] = False
+            cn_change = compute_cn_change(df=df[allele_columns], tree=input_tree, normal_name=normal_name)
+            
+            df.loc[(cn_change < 0).any(axis=1), 'is_loss'] = True
+            df.loc[(cn_change > 0).any(axis=1), 'is_gain'] = True
+
+            is_normal = ~df.unstack('sample_id')[['is_loss', 'is_gain', 'is_wgd']].any(axis=1)
+            is_normal.name = 'is_normal'
+            mrca = [x for x in input_tree.root.clades if x.name != normal_name][0].name
+            is_clonal = ~df.loc[df.index.get_level_values('sample_id')!=mrca].unstack('sample_id')[['is_loss', 'is_gain', 'is_wgd']].any(axis=1)
+            is_clonal.name = 'is_clonal'
+
+            df = df.drop(['is_normal', 'is_clonal'], axis=1, errors='ignore')
+            df = (df
+                    .join(is_normal, how='inner')
+                    .reorder_levels(['sample_id', 'chrom', 'start', 'end'])
+                    .sort_index()
+                    .join(is_clonal, how='inner')
+                    .reset_index())
+            df = (df
+                    .set_index(['sample_id', 'chrom', 'start', 'end'])
+                    .sort_index())
+
+
+
 
     if hide_normal_chromosomes:
         df = df.join(df.groupby('chrom')['is_normal'].all().to_frame('hide'))
@@ -113,7 +139,7 @@ def plot_cn_profiles(
 
     if nsamp > 20:
         logger.warn('More than 20 samples were provided. Creating the copy number tracks will take '
-                    'a long time to process and might crash.\nBest to use plot_tree instead.')
+                    'a long time to process and might crash. Best to use plot_cn_heatmap instead')
 
     df.reset_index(['start','end'], inplace=True)
 
@@ -145,10 +171,10 @@ def plot_cn_profiles(
     df = df.reset_index().set_index(['sample_id', 'chrom', 'start', 'end'])
 
     if plot_summary or plot_subclonal_summary:
-        agg_events = core.compute_cn_change(df=df, tree=input_tree, normal_name=normal_name)
+        agg_events = compute_cn_change(df=df[allele_columns], tree=input_tree, normal_name=normal_name)
         agg_events = agg_events.groupby(["chrom", "start", "end"], observed=True).sum()
 
-        agg_events.loc[:, ['start_pos', 'end_pos', 'small_segment']
+        agg_events[['start_pos', 'end_pos', 'small_segment']
                        ] = df.loc[samples[0], ['start_pos', 'end_pos', 'small_segment']]
 
         mrca = [x for x in input_tree.root.clades if x.name != normal_name][0].name
@@ -172,7 +198,7 @@ def plot_cn_profiles(
         if sample == normal_name:
             clade_colors[sample] = COL_MARKER_NORMAL
 
-    # %% define plot width and height and create figure
+    # define plot width and height and create figure
     track_width = nsegs.sum() * 0.2 * track_width_scale
     if input_tree is not None:
         tree_width = 2.5 * tree_width_scale ## in figsize
@@ -183,7 +209,7 @@ def plot_cn_profiles(
     plotheight = 4 * 0.2 * nrows * height_scale
     plotwidth = tree_width + track_width
     tree_width_ratio = tree_width / plotwidth
-    fig = plt.figure(figsize=(plotwidth, plotheight), constrained_layout=True)
+    fig = plt.figure(figsize=(min(250, plotwidth), min(250, plotheight)), constrained_layout=True)
     if input_tree is None:
         gs = fig.add_gridspec(nrows, 1)
         cn_axes = [fig.add_subplot(gs[i]) for i in range(0, nrows)]
@@ -199,6 +225,7 @@ def plot_cn_profiles(
         plot_tree(input_tree, 
                   ax=tree_ax,
                   title=title,
+                  normal_name=normal_name,
                   label_func=lambda x: '',
                   label_colors=clade_colors,
                   show_branch_support=show_branch_support,
@@ -232,9 +259,11 @@ def plot_cn_profiles(
                          show_events_in_cn=show_events_in_cn)
 
     if plot_clonal_summary:
+        cur = mrca_df.copy()
+        cur[['is_clonal', 'is_normal', 'is_gain', 'is_loss', 'is_wgd']] = False
         _plot_cn_profile(ax=cn_axes[nsamp],
                          label='clonal\nchanges',
-                         data=mrca_df.copy(),
+                         data=cur,
                          mincn=mrca_df[allele_columns].min().min()-1,
                          maxcn=mrca_df[allele_columns].max().max()+1,
                          alleles=allele_columns,
@@ -245,9 +274,11 @@ def plot_cn_profiles(
         cn_axes[nsamp].get_xaxis().set_visible(not (plot_summary or plot_subclonal_summary))
 
     if plot_summary:
+        cur = agg_events.copy()
+        cur[['is_clonal', 'is_normal', 'is_gain', 'is_loss', 'is_wgd']] = False
         _plot_cn_profile(ax=cn_axes[-1],
                          label='all\nchanges',
-                         data=agg_events.copy(),
+                         data=cur,
                          mincn=agg_events[allele_columns].min().min()-1,
                          maxcn=agg_events[allele_columns].max().max()+1,
                          alleles=allele_columns,
@@ -258,9 +289,12 @@ def plot_cn_profiles(
 
     if plot_subclonal_summary:
         agg_events.loc[:, allele_columns] = agg_events[allele_columns] - mrca_df[allele_columns]
+        cur = agg_events.copy()
+        cur[['is_clonal', 'is_normal', 'is_gain', 'is_loss', 'is_wgd']] = False
+
         _plot_cn_profile(ax=cn_axes[-1 - int(plot_summary)],
                          label='subclonal\nchanges',
-                         data=agg_events.copy(),
+                         data=cur,
                          type='summary',
                          mincn=agg_events[allele_columns].min().min()-1,
                          maxcn=agg_events[allele_columns].max().max()+1,
@@ -387,7 +421,7 @@ def _plot_cn_profile(ax, label, data, mincn, maxcn, alleles, type='sample',
     chr_label_pos = chr_ends
     chr_label_pos.loc[:] = np.roll(chr_label_pos.values, 1)
     chr_label_pos.iloc[0] = seg_bound_first
-    for chrom, pos in chr_label_pos.iteritems():
+    for chrom, pos in chr_label_pos.items():
         if chr_label_func is not None:
             chromtxt = chr_label_func(chrom)
         else:
@@ -534,7 +568,6 @@ def plot_tree(input_tree,
             ) from None
 
     import matplotlib.collections as mpcollections
-
     if ax is None:
         nsamp = len(list(input_tree.find_clades()))
         plot_height = height_scale * nsamp * 0.25
@@ -542,7 +575,8 @@ def plot_tree(input_tree,
                             for leaf in input_tree.get_terminals()])
         plot_width = 5 + np.max([0, width_scale * np.log10(max_leaf_to_root_distances / 100) * 5])
 
-        fig, ax = plt.subplots(figsize=(plot_width, plot_height))
+        # maximum figure size is 250x250 inches
+        fig, ax = plt.subplots(figsize=(min(250, plot_width), min(250, plot_height)))
 
     label_func=label_func if label_func is not None else lambda x: x
 
@@ -697,7 +731,7 @@ def plot_tree(input_tree,
                 ax.scatter(x_here, y_here, s=marker_size, c=marker_col, zorder=3)
 
         # Add node/taxon labels
-        label = label_func(str(clade))
+        label = label_func(str(clade.name))
         ax_scale = ax.get_xlim()[1] - ax.get_xlim()[0]
 
         if label not in (None, clade.__class__.__name__) and \
@@ -806,50 +840,62 @@ def plot_tree(input_tree,
 
 
 def plot_cn_heatmap(input_df, final_tree=None, y_posns=None, cmax=8, total_copy_numbers=False,
-                    alleles='total', tree_width_ratio=1, cbar_width_ratio=0.05, figsize=(20, 10),
-                    tree_line_width=0.5, tree_marker_size=0, show_internal_nodes=False,
-                    tree_label_colors=None, tree_label_func=None, cmap='coolwarm'):
+                    alleles=['cn_a', 'cn_b'], tree_width_ratio=1, cbar_width_ratio=0.05, figsize=(20, 10),
+                    tree_line_width=0.5, tree_marker_size=0, show_internal_nodes=False, title='',
+                    tree_label_colors=None, tree_label_func=None, cmap='coolwarm', normal_name='diploid',
+                    ignore_segment_lengths=False):
 
-    if show_internal_nodes:
-        cur_sample_labels = np.array([x.name for x in list(final_tree.find_clades()) if x.name is not None])
-    else:
-        cur_sample_labels = np.array([x.name for x in final_tree.get_terminals()])
-    if len(np.intersect1d(cur_sample_labels, input_df.index.get_level_values('sample_id').unique())) != len(cur_sample_labels):
-        raise MEDICCPlotError("tree nodes and labels in dataframe are not the same")
+    input_df = input_df[alleles].copy()
+    # if len(np.intersect1d(cur_sample_labels, input_df.index.get_level_values('sample_id').unique())) != len(cur_sample_labels):
+    #     raise MEDICCPlotError("tree nodes and labels in dataframe are not the same")
 
     if not isinstance(alleles, list) and not isinstance(alleles, tuple):
         alleles = [alleles]
     nr_alleles = len(alleles)
 
-    cmax = min(cmax, np.max(input_df.values.astype(int)))
+    cmax = min(cmax, np.max(input_df[alleles].values.astype(int)))
 
     if final_tree is None:
         fig, axs = plt.subplots(figsize=figsize, ncols=1+nr_alleles, sharey=False,
                                 gridspec_kw={'width_ratios': nr_alleles*[1] + [cbar_width_ratio]})
 
+        cur_sample_labels = (input_df.index.get_level_values('sample_id').unique())
         if y_posns is None:
-            y_posns = {s: i for i, s in enumerate(np.sort(cur_sample_labels))}
+            y_posns = {s: i for i, s in enumerate(cur_sample_labels)}
 
         cn_axes = axs[:-1]
+        cn_axes[0].set_title(title, x=0, y=1, ha='left', va='bottom', pad=20,
+                             fontweight='bold', fontsize=16, zorder=10)
     else:
         fig, axs = plt.subplots(figsize=figsize, ncols=2+nr_alleles, sharey=False,
                                 gridspec_kw={'width_ratios': [tree_width_ratio] + nr_alleles*[1] + [cbar_width_ratio]})
         tree_ax = axs[0]
         cn_axes = axs[1:-1]
 
-        y_posns = {k.name:v for k, v in _get_y_positions(final_tree, adjust=show_internal_nodes).items()}
+        if show_internal_nodes:
+            cur_sample_labels = np.array([x.name for x in list(final_tree.find_clades()) if x.name is not None])
+        else:
+            cur_sample_labels = np.array([x.name for x in final_tree.get_terminals()])
+
+        y_posns = {k.name:v for k, v in _get_y_positions(final_tree, adjust=show_internal_nodes, normal_name=normal_name).items()}
         
-        _ = plot_tree(final_tree, ax=tree_ax,
+        _ = plot_tree(final_tree, ax=tree_ax, normal_name=normal_name,
                       label_func=tree_label_func if tree_label_func is not None else lambda x: '',
                       hide_internal_nodes=(not show_internal_nodes), show_branch_lengths=False, show_events=False,
                       line_width=tree_line_width, marker_size=tree_marker_size,
-                      title='', label_colors=tree_label_colors)
+                      title=title, label_colors=tree_label_colors)
         tree_ax.set_axis_off()
         tree_ax.set_axis_off()
         fig.set_constrained_layout_pads(w_pad=0, h_pad=0, hspace=0.0, wspace=100)
-
     cax = axs[-1]
 
+    gaps = (input_df.loc[cur_sample_labels[0]].eval('start') - 
+            np.roll(input_df.loc[cur_sample_labels[0]].eval('end'), 1)).values
+    total_gaps = gaps[gaps>0].sum()
+    if total_gaps > 1e8:
+        logger.warn(f"Total of {total_gaps:.1e} bp gaps in the segmentation. These missing "
+                    "segments are not reflected in the plot!")
+
     ind = [y_posns.get(x, -1) for x in cur_sample_labels]
     cur_sample_labels = cur_sample_labels[np.argsort(ind)]
     color_norm = mcolors.TwoSlopeNorm(vmin=0, vcenter=(2 if total_copy_numbers else 1), vmax=cmax)
@@ -857,10 +903,15 @@ def plot_cn_heatmap(input_df, final_tree=None, y_posns=None, cmax=8, total_copy_
     chr_ends = input_df.loc[cur_sample_labels[0]].copy()
     chr_ends['end_pos'] = np.cumsum([1]*len(chr_ends))
     chr_ends = chr_ends.reset_index().groupby('chrom').max()['end_pos']
-    chr_ends.dropna(inplace=True)
+    chr_ends = chr_ends.dropna()
+    chr_ends = chr_ends.astype(int)
 
-    x_pos = np.append([0], np.cumsum(input_df.loc[cur_sample_labels].astype(int).unstack(
-        'sample_id').loc[:, (alleles[0])].loc[:, cur_sample_labels].eval('end-start').values))
+
+    if ignore_segment_lengths:
+        x_pos = np.arange(len(input_df.loc[cur_sample_labels].astype(int).unstack('sample_id'))+1)
+    else:
+        x_pos = np.append([0], np.cumsum(input_df.loc[cur_sample_labels].astype(int).unstack(
+            'sample_id').loc[:, (alleles[0])].loc[:, cur_sample_labels].eval('end+1-start').values))
     y_pos = np.arange(len(cur_sample_labels)+1)+0.5
 
     for ax, allele in zip(cn_axes, alleles):
@@ -870,7 +921,7 @@ def plot_cn_heatmap(input_df, final_tree=None, y_posns=None, cmax=8, total_copy_
                         cmap=cmap,
                         norm=color_norm)
 
-        for _, line in chr_ends.iteritems():
+        for _, line in chr_ends.items():
             ax.axvline(x_pos[line], color='black', linewidth=0.75)
         
         xtick_pos = np.append([0], x_pos[chr_ends.values][:-1])
@@ -903,5 +954,31 @@ def plot_cn_heatmap(input_df, final_tree=None, y_posns=None, cmax=8, total_copy_
     return fig
 
 
+def compute_cn_change(df, tree, normal_name='diploid'):
+    """Compute the copy-number changes per segment in all branches
+
+    Args:
+        df (pandas.DataFrame): DataFrame containing the copy-numbers of samples and internal nodes
+        tree (Bio.Phylo.Tree): Phylogenetic tree
+        normal_name (str, optional): Name of normal sample. Defaults to 'diploid'.
+
+    Returns:
+        pandas.DataFrame: DataFrame containing the copy-number changes
+    """    
+    cn_change = df.copy()
+    alleles = cn_change.columns
+    for allele in alleles:
+        cn_change[allele] = cn_change[allele].astype('int')
+
+    clades = [clade for clade in tree.find_clades(order = "postorder") if clade.name is not None and clade.name != normal_name]
+    for clade in clades:
+        for child in clade.clades:
+            cn_change.loc[child.name, alleles] = cn_change.loc[child.name, alleles].values - cn_change.loc[clade.name, alleles].values
+    cn_change.loc[clades[-1].name, alleles] = cn_change.loc[clades[-1].name, alleles].values - cn_change.loc[normal_name, alleles].values
+    cn_change.loc[normal_name, alleles] = 0
+
+    return cn_change
+
+
 class MEDICCPlotError(Exception):
     pass
