@@ -9,14 +9,16 @@ import pandas as pd
 logger = logging.getLogger('medicc.io')
 
 def set_sequences_on_tree_from_df(tree: Bio.Phylo.BaseTree, df: pd.DataFrame, clear_before=True):
-    """Set sequences on tree from dataframe
+    """DEPRECATED!
+    
+    Set sequences on tree from dataframe
 
     Args:
         tree (Bio.Phylo.BaseTree): Tree to set sequences on
         df (pd.DataFrame): DataFrame with copy number information
         clear_before (bool, optional): Clear old sequences. Defaults to True.
     """
-
+    raise DeprecationWarning("This function is deprecated and is not used anymore. ")
 
     if not hasattr(tree.root, 'sequences'):
         tree = tree.as_phyloxml()
@@ -129,7 +131,7 @@ def total_pdm_from_parallel_pdms(sample_labels, parallel_pdms):
     return total_pdm
 
 
-def create_diploid_fsa(fst):
+def create_diploid_fsa(fst, total_copy_numbers=False):
 
     # Create diploid FSA
     diploid_fsa = fstlib.Fst()
@@ -138,7 +140,10 @@ def create_diploid_fsa(fst):
     diploid_fsa.add_state()
     diploid_fsa.set_start(0)
     diploid_fsa.set_final(0, 0)
-    diploid_fsa.add_arc(0, ('1', '1', 0, 0))
+    if total_copy_numbers:
+        diploid_fsa.add_arc(0, ('2', '2', 0, 0))
+    else:
+        diploid_fsa.add_arc(0, ('1', '1', 0, 0))
     diploid_fsa.add_arc(0, ('X', 'X', 0, 0))
 
     return diploid_fsa
